import logging
from datetime import datetime, timezone
from typing import Optional

from .binance import Binance
from .bit2me import Bit2Me
from .bitvavo import Bitvavo
from .coinbase import Coinbase
from .coingecko import CoinGecko
from .coinpaprika import CoinPaprika
from .kraken import Kraken
from .mempool import Mempool
from .price_timeseries import PriceTimeSeries
from .service import Service

logger = logging.getLogger(__name__)


class Price:
    def __init__(
        self,
        fiat: str = "eur",
        days_ago: int = 1,
        min_refresh_time: int = 120,
        interval: str = "1h",
        ohlcv_interval: str = "1h",
        service: str = "mempool",
        enable_ohlcv: bool = False,
        enable_ohlc: bool = False,
        enable_timeseries: bool = True,
    ) -> None:
        self.days_ago = days_ago
        self.interval = interval
        self.available_services = [
            "mempool",
            "coingecko",
            "coinpaprika",
            "kraken",
            "binance",
            "coinbase",
            "bitvavo",
            "bit2me",
        ]
        if service not in self.available_services:
            raise ValueError("Wrong service!")
        self.services: dict[str, Service] = {}
        self.service = service
        self.enable_ohlcv = enable_ohlcv or enable_ohlc
        self.set_service(
            service,
            fiat,
            interval,
            days_ago,
            enable_ohlc,
            enable_timeseries,
            enable_ohlcv,
        )
        self.min_refresh_time = min_refresh_time  # seconds
        self.fiat = fiat
        self.enable_ohlc = enable_ohlc
        self.enable_ohlcv = enable_ohlcv
        self.enable_timeseries = enable_timeseries

    def set_next_service(self, next_service: Optional[str] = None) -> None:
        fiat = self.fiat
        service_name = self.service
        interval = self.interval
        days_ago = self.days_ago
        enable_ohlc = self.enable_ohlc
        enable_ohlcv = self.enable_ohlcv
        enable_timeseries = self.enable_timeseries
        if next_service is None:
            rotation = [
                "mempool",
                "coingecko",
                "coinpaprika",
                "kraken",
                "binance",
                "coinbase",
                "bitvavo",
                "bit2me",
            ]
            try:
                current_index = rotation.index(service_name)
            except ValueError:
                current_index = -1
            next_service = rotation[(current_index + 1) % len(rotation)]

        self.set_service(
            next_service,
            fiat,
            interval,
            days_ago,
            enable_ohlc,
            enable_timeseries,
            enable_ohlcv,
        )

    def set_service(
        self,
        service_name: str,
        fiat: str,
        interval: str,
        days_ago: int,
        enable_ohlc: bool,
        enable_timeseries: bool,
        enable_ohlcv: bool,
    ) -> None:
        if service_name in self.services:
            self.service = service_name
            return

        service_instance: Optional[Service] = None
        if service_name == "coingecko":
            service_instance = CoinGecko(
                fiat,
                whichcoin="bitcoin",
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "coinpaprika":
            service_instance = CoinPaprika(
                fiat,
                whichcoin="btc-bitcoin",
                interval=interval,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "mempool":
            service_instance = Mempool(
                fiat,
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "kraken":
            service_instance = Kraken(
                fiat,
                base_asset="BTC",
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "binance":
            service_instance = Binance(
                fiat,
                base_asset="BTC",
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "coinbase":
            service_instance = Coinbase(
                fiat,
                base_asset="BTC",
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "bitvavo":
            service_instance = Bitvavo(
                fiat,
                base_asset="BTC",
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )
        elif service_name == "bit2me":
            service_instance = Bit2Me(
                fiat,
                base_asset="BTC",
                interval=interval,
                days_ago=days_ago,
                enable_ohlc=enable_ohlc,
                enable_timeseries=enable_timeseries,
                enable_ohlcv=enable_ohlcv,
            )

        if service_instance is None:
            raise ValueError(f"Unsupported service '{service_name}'")

        self.service = service_name
        self.services[service_name] = service_instance

    def _fetch_prices(self):
        """Fetch prices and OHLCV data from Service."""
        if self.service not in self.services:
            self.set_next_service()
        self.services[self.service].update()

    def refresh(self):
        """Refresh the price data if necessary."""
        count = 0
        refresh_sucess = self.update_service()
        old_service_name = self.service
        while not refresh_sucess and count < 3:
            self.set_next_service()
            refresh_sucess = self.update_service()
            count += 1

        self.set_next_service(next_service=old_service_name)
        return refresh_sucess

    def update_service(self):
        now = datetime.now(timezone.utc)
        current_time = now.timestamp()

        if (
            "timestamp" in self.price
            and current_time - self.price["timestamp"] < self.min_refresh_time
        ):
            return True

        logger.info("Fetching price data...")
        try:
            self._fetch_prices()
            return True
        except Exception as e:
            logger.warning(f"Failed to fetch from  {self.service}: {str(e)}")
        return False

    def get_price_list(self):
        return self.services[self.service].get_price_list()

    def get_timeseries_list(self):
        return self.get_price_list()

    @property
    def timeseries_stack(self):
        return self.get_price_list()

    @property
    def price(self):
        return self.services[self.service].get_price()

    @property
    def ohlc(self):
        return self.services[self.service].ohlc

    @property
    def ohlcv(self):
        return self.services[self.service].ohlcv

    def set_days_ago(self, days_ago: int) -> None:
        self.days_ago = days_ago
        for service in self.services:
            self.services[service].days_ago = days_ago

    def get_price_change(self) -> str:
        return self.services[self.service].get_price_change()

    def get_fiat_currency(self) -> str:
        return self.services[self.service].fiat.upper()

    def get_fiat_symbol(self) -> str:
        """Return the symbol of the fiat currency."""
        fiat = self.get_fiat_currency().lower()
        symbols = {
            "eur": "€",
            "usd": "$",
            "gbp": "£",
            "jpy": "¥",
            "chf": "CHF",
            "cad": "C$",
            "aud": "A$",
            "mxn": "$",
            "brl": "R$",
            "inr": "₹",
            "cny": "¥",
            "cnh": "¥",
            "sek": "kr",
            "nzd": "NZ$",
        }
        return symbols.get(fiat, fiat)

    def get_fiat_price(self) -> float:
        return self.price["fiat"]

    def get_usd_price(self) -> float:
        return self.price["usd"]

    def get_sats_per_fiat(self) -> float:
        return 1e8 / self.price["fiat"]

    def get_sats_per_usd(self) -> float:
        return 1e8 / self.price["usd"]

    def get_timestamp(self) -> float:
        return self.price["timestamp"]

    def get_price_now(self) -> str:
        self.update_service()
        price_now = self.price["fiat"]
        return f"{price_now:,.0f}" if price_now > 1000 else f"{price_now:.5g}"

    @property
    def timeseries(self) -> PriceTimeSeries:
        return self.services[self.service].price_history



import logging
from datetime import datetime, timezone
from typing import Optional

import pandas as pd
from pycoingecko import CoinGeckoAPI

from .service import Service

logger = logging.getLogger(__name__)


class CoinGecko(Service):
    def __init__(
        self,
        fiat,
        whichcoin="bitcoin",
        days_ago=1,
        enable_ohlc=True,
        enable_timeseries=True,
        enable_ohlcv=True,
    ):
        self.cg = CoinGeckoAPI()
        self.whichcoin = whichcoin
        self.initialize(
            fiat,
            days_ago=days_ago,
            enable_ohlc=enable_ohlc,
            enable_timeseries=enable_timeseries,
            enable_ohlcv=enable_ohlcv,
        )
        self.name = "coingecko"

    def get_current_price(self, currency) -> Optional[float]:
        """Fetch the current price for the given currency from CoinGecko."""
        normalized_currency = currency.lower()
        try:
            return float(
                self.cg.get_coins_markets(normalized_currency, ids=self.whichcoin)[0][
                    "current_price"
                ]
            )
        except (IndexError, KeyError):
            logger.error(f"Failed to retrieve price for {self.whichcoin} in {currency}")
            return None

    def get_exchange_usd_price(self, exchange):
        """Fetch the USD price for the given exchange."""
        try:
            raw_data = self.cg.get_exchanges_tickers_by_id(
                exchange, coin_ids=self.whichcoin
            )
            ticker = raw_data["tickers"][0]
            if ticker["target"] != "USD":
                logger.info("Not USD, could not get price.")
                return None
            return float(ticker["last"])
        except (IndexError, KeyError):
            logger.error(f"Failed to retrieve exchange price for {exchange}")
            return None

    def interval_to_seconds(self) -> int:
        return 60 * 5

    def get_history_price(self, currency, existing_timestamp=None) -> dict:
        normalized_currency = currency.lower()
        if existing_timestamp:
            now = datetime.now(timezone.utc)
            intervals = self.interval_to_seconds()
            start_time = datetime.fromtimestamp(
                existing_timestamp[-1] + intervals, tz=timezone.utc
            )
            raw_data = self.cg.get_coin_market_chart_range_by_id(
                self.whichcoin,
                normalized_currency,
                from_timestamp=start_time.timestamp(),
                to_timestamp=now.timestamp(),
            )
        else:
            raw_data = self.cg.get_coin_market_chart_by_id(
                self.whichcoin, normalized_currency, self.days_ago
            )
        return raw_data

    def update_price_history(self, currency) -> None:
        """Fetch historical prices from CoinGecko."""
        logger.info(f"Getting historical data for {self.days_ago} days")
        existing_timestamp = self.price_history.get_timestamp_list()
        raw_data = self.get_history_price(
            currency, existing_timestamp=existing_timestamp
        )
        timeseries = raw_data.get("prices", [])
        for price in timeseries:
            dt = datetime.fromtimestamp(float(price[0]) / 1000, tz=timezone.utc)
            self.price_history.add_price(dt, float(price[1]))

    def get_ohlcv(self, currency, existing_timestamp=None) -> pd.DataFrame:
        """Fetch OHLCV data based on the number of days ago."""
        normalized_currency = currency.lower()
        time_ranges = [1, 7, 14, 30, 90, 180, 365]
        duration = next((d for d in time_ranges if self.days_ago <= d), "max")
        raw_ohlc = self.cg.get_coin_ohlc_by_id(
            self.whichcoin, normalized_currency, duration
        )

        timeseries = [
            {
                "time": datetime.fromtimestamp(ohlc[0] / 1000, tz=timezone.utc),
                "ohlcv": ohlc[1:] + [0],
            }
            for ohlc in raw_ohlc
            if (
                datetime.fromtimestamp(raw_ohlc[-1][0] / 1000, tz=timezone.utc)
                - datetime.fromtimestamp(ohlc[0] / 1000, tz=timezone.utc)
            ).days
            <= self.days_ago
        ]

        df = pd.DataFrame(
            [ohlcv["ohlcv"] for ohlcv in timeseries],
            columns=["Open", "High", "Low", "Close", "Volume"],
        )
        df.index = [ohlcv["time"] for ohlcv in timeseries]

        if existing_timestamp:
            cutoff = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
            df = df[df.index > cutoff]

        return df

    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        """Fetch OHLC data based on the number of days ago."""
        ohlcv_df = self.get_ohlcv(currency, existing_timestamp)
        ohlc_df = ohlcv_df.drop(columns=["Volume"])
        return ohlc_df



import logging
from datetime import datetime, timedelta, timezone
from typing import Any, Optional

import pandas as pd

from .service import Service

logger = logging.getLogger(__name__)

CCXT_MODULE = None
try:
    import ccxt

    CCXT_MODULE = "ccxt"
except ImportError:  # pragma: no cover
    ccxt = None  # type: ignore


class Kraken(Service):
    def __init__(
        self,
        fiat: str,
        base_asset: str = "BTC",
        interval: str = "1h",
        days_ago: int = 1,
        enable_ohlc: bool = True,
        enable_timeseries: bool = True,
        enable_ohlcv: bool = True,
    ):
        self.base_asset = base_asset.upper()
        self.exchange: Optional[Any] = ccxt.kraken() if CCXT_MODULE else None  # type: ignore[attr-defined]
        self.initialize(
            fiat,
            interval=interval,
            days_ago=days_ago,
            enable_ohlc=enable_ohlc,
            enable_timeseries=enable_timeseries,
            enable_ohlcv=enable_ohlcv,
        )
        self.name = "kraken"

    def _get_symbol(self, currency: str) -> str:
        quote = currency.upper()
        return f"{self.base_asset}/{quote}"

    def interval_to_seconds(self) -> int:
        unit_multipliers = {"m": 60, "h": 3600, "d": 86400}
        try:
            value, unit = int(self.interval[:-1]), self.interval[-1]
            if unit not in unit_multipliers:
                raise ValueError
            return value * unit_multipliers[unit]
        except (ValueError, IndexError) as exc:  # pragma: no cover
            raise ValueError(f"Invalid interval format {self.interval}") from exc

    def get_current_price(self, currency: str) -> Optional[float]:
        if self.exchange is None:
            return None
        symbol = self._get_symbol(currency)
        try:
            ticker = self.exchange.fetch_ticker(symbol)
            last_price = ticker.get("last") or ticker.get("close")
            return float(last_price) if last_price is not None else None
        except Exception as exc:  # pragma: no cover - network or API errors
            logger.exception(f"Failed to fetch current price for {symbol}: {exc}")
            return None

    def _calculate_since(
        self, existing_timestamp: Optional[list[float]]
    ) -> Optional[int]:
        if not existing_timestamp:
            start = datetime.now(timezone.utc) - timedelta(days=self.days_ago)
            return int(start.timestamp() * 1000)
        step_seconds = self.interval_to_seconds()
        last_timestamp = existing_timestamp[-1] + step_seconds
        return int(last_timestamp * 1000)

    def get_history_price(
        self, currency: str, existing_timestamp: Optional[list[float]] = None
    ) -> list[list[float]]:
        if self.exchange is None:
            return []
        symbol = self._get_symbol(currency)
        since = self._calculate_since(existing_timestamp)
        try:
            return self.exchange.fetch_ohlcv(
                symbol,
                timeframe=self.interval,
                since=since,
            )
        except Exception as exc:  # pragma: no cover - network or API errors
            logger.exception(f"Failed to fetch historical prices for {symbol}: {exc}")
            return []

    def update_price_history(self, currency: str) -> None:
        if self.exchange is None:
            return
        logger.info(
            "Getting historical data for a %s interval from Kraken", self.interval
        )
        existing_timestamp = self.price_history.get_timestamp_list()
        ohlcv_data = self.get_history_price(currency, existing_timestamp)
        for candle in ohlcv_data:
            if len(candle) < 5:
                continue
            timestamp_ms, _, _, _, close_price = candle[:5]
            dt = datetime.fromtimestamp(timestamp_ms / 1000, tz=timezone.utc)
            self.price_history.add_price(dt, float(close_price))

    def get_ohlcv(self, currency: str, existing_timestamp=None) -> pd.DataFrame:
        if self.exchange is None:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])
        symbol = self._get_symbol(currency)
        try:
            since = self._calculate_since(existing_timestamp)
            ohlcv_data = self.exchange.fetch_ohlcv(
                symbol, timeframe=self.interval, since=since
            )
        except Exception as exc:  # pragma: no cover - network or API errors
            logger.exception(f"Failed to fetch OHLCV data for {symbol}: {exc}")
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])

        if not ohlcv_data:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])

        times = [
            datetime.fromtimestamp(candle[0] / 1000, tz=timezone.utc)
            for candle in ohlcv_data
        ]
        values = [candle[1:6] for candle in ohlcv_data]
        df = pd.DataFrame(
            values, columns=["Open", "High", "Low", "Close", "Volume"], index=times
        )

        if existing_timestamp:
            cutoff = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
            df = df[df.index > cutoff]
        return df

    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        """Fetch OHLC data based on the number of days ago."""
        ohlcv_df = self.get_ohlcv(currency, existing_timestamp)
        ohlc_df = ohlcv_df.drop(columns=["Volume"])
        return ohlc_df



import unittest
from datetime import datetime, timezone
from unittest.mock import patch

import pandas as pd
from pytest import approx

from btcpriceticker.bit2me import Bit2Me


class TestBit2Me(unittest.TestCase):
    def test_get_current_price_converts_with_rate(self):
        with (
            patch.object(Bit2Me, "_get_usd_price", return_value=50000.0),
            patch.object(Bit2Me, "_get_fiat_rate", return_value=0.9),
        ):
            service = Bit2Me("EUR")
            price = service.get_current_price("EUR")

        self.assertEqual(price, 45000.0)

    def test_get_history_price_from_chart(self):
        chart_data = [
            [1712497400000, "0.00005", "0.9"],
            [1712498400000, "0.00004", "0.9"],
        ]
        with patch.object(Bit2Me, "_chart", return_value=chart_data):
            service = Bit2Me("EUR")
            history = service.get_history_price("EUR")

        prices = [entry[1] for entry in history]
        assert prices[0] == approx(18000.0)
        assert prices[1] == approx(22500.0)

    def test_update_price_history_stores_converted_values(self):
        chart_data = [
            [1712497400000, "0.00005", "0.8"],
            [1712498400000, "0.00005", "0.82"],
        ]
        with patch.object(Bit2Me, "_chart", return_value=chart_data):
            service = Bit2Me("EUR")
            service.update_price_history("EUR")

        prices = service.price_history.data["price"].tolist()
        assert prices[0] == approx(16000.0)
        assert prices[1] == approx(16400.0)

    def test_fetch_ohlc_point_converts_prices(self):
        service = Bit2Me("EUR")
        target_dt = datetime(2025, 1, 1, tzinfo=timezone.utc)
        with patch.object(
            Bit2Me,
            "_request",
            side_effect=[
                {"open": "100.0", "high": "110.0", "low": "90.0", "close": "105.0"},
                [{"fiat": {"EUR": "0.9"}}],
            ],
        ):
            result = service._fetch_ohlc_point("1H", "EUR", target_dt)

        self.assertIsNotNone(result)
        assert result is not None  # for type checkers
        ts, data = result
        self.assertEqual(ts, target_dt)
        assert data["Open"] == approx(90.0)
        assert data["High"] == approx(99.0)
        assert data["Low"] == approx(81.0)
        assert data["Close"] == approx(94.5)

    def test_get_ohlc_returns_multiple_rows(self):
        service = Bit2Me("EUR", interval="12h")
        service.days_ago = 1

        with patch.object(Bit2Me, "_fetch_ohlc_point", autospec=True) as mock_fetch:

            def fake_fetch(self, timeframe, currency, target_dt):
                base = target_dt.timestamp()
                return (
                    target_dt,
                    {
                        "Open": base,
                        "High": base + 1,
                        "Low": base - 1,
                        "Close": base + 0.5,
                    },
                )

            mock_fetch.side_effect = fake_fetch
            df = service.get_ohlc("EUR")

        self.assertIsInstance(df, pd.DataFrame)
        self.assertFalse(df.empty)
        self.assertListEqual(list(df.columns), ["Open", "High", "Low", "Close"])
        self.assertEqual(len(df), 2)

    def test_get_ohlcv_adds_volume(self):
        service = Bit2Me("EUR")

        with patch.object(Bit2Me, "_fetch_ohlc_point", autospec=True) as mock_fetch:

            def fake_fetch(self, timeframe, currency, target_dt):
                return (
                    target_dt,
                    {
                        "Open": 100.0,
                        "High": 110.0,
                        "Low": 90.0,
                        "Close": 105.0,
                    },
                )

            mock_fetch.side_effect = fake_fetch
            df = service.get_ohlcv("EUR")

        self.assertIsInstance(df, pd.DataFrame)
        self.assertFalse(df.empty)
        self.assertListEqual(
            list(df.columns), ["Open", "High", "Low", "Close", "Volume"]
        )
        assert df.iloc[0]["Volume"] == 0.0


if __name__ == "__main__":
    unittest.main()



import logging
from datetime import datetime, timedelta, timezone
from typing import Any, Optional

import pandas as pd

from .service import Service

logger = logging.getLogger(__name__)

COINPAPRIKA_MODULE = None
try:
    from coinpaprika import client as Coinpaprika

    COINPAPRIKA_MODULE = "coinpaprika"
except ImportError:
    pass


class CoinPaprika(Service):
    def __init__(
        self,
        fiat,
        whichcoin="btc-bitcoin",
        days_ago=1,
        interval="1h",
        enable_ohlc=False,
        enable_timeseries=True,
        enable_ohlcv=False,
    ):
        self.api_client: Optional[Any] = (
            Coinpaprika.Client() if COINPAPRIKA_MODULE else None
        )
        self.whichcoin = whichcoin
        interval = "1h"
        self.initialize(
            fiat,
            interval=interval,
            days_ago=days_ago,
            enable_ohlc=enable_ohlc,
            enable_timeseries=enable_timeseries,
            enable_ohlcv=enable_ohlcv,
        )
        self.coins: Optional[list[dict[str, Any]]] = None
        self.name = "coinpaprika"

    def get_coin(
        self, name: Optional[str] = None, symbol: Optional[str] = None
    ) -> Optional[dict[str, Any]]:
        if self.api_client is None:
            return None
        if self.coins is None:
            self.coins = self.api_client.coins()
        for coin in self.coins:
            if name and coin["name"] == name:
                return coin
            if symbol and coin["symbol"] == symbol:
                return coin
        return None

    def interval_to_seconds(self) -> int:
        """Convert a time interval string to seconds."""
        unit_multipliers = {"m": 60, "h": 3600, "d": 86400}

        try:
            value, unit = int(self.interval[:-1]), self.interval[-1]
            if unit in unit_multipliers:
                return value * unit_multipliers[unit]
            else:
                raise ValueError(f"Invalid interval format {self.interval}")
        except (ValueError, IndexError) as e:
            raise ValueError(f"Invalid interval format {self.interval}") from e

    def get_current_price(self, currency: str = "USD") -> Optional[float]:
        """Fetch the current price from Coinpaprika."""
        if not self.api_client:
            return None
        try:
            ticker = self.api_client.ticker(self.whichcoin, quotes=currency.upper())
            return float(ticker["quotes"][currency.upper()]["price"])
        except Exception as e:
            logger.exception(f"Failed to fetch current price: {e}")
            return None

    def get_exchange_usd_price(
        self, exchange: str, pair: str, currency: str = "USD"
    ) -> Optional[float]:
        """Fetch the USD price for a given exchange and pair."""
        if not self.api_client:
            return None
        try:
            markets = self.api_client.exchange_markets(exchange, quotes=currency)
            for market in markets:
                if market["pair"] == pair:
                    return float(market["quotes"][currency]["price"])
            logger.info("Not USD, could not get price.")
            return None
        except Exception as e:
            logger.exception(f"Failed to fetch exchange USD price: {e}")
            return None

    def calculate_start_date(
        self, interval: str, existing_timestamp: Optional[list[float]] = None
    ) -> str:
        now = datetime.now(timezone.utc)
        intervals = self.interval_to_seconds()

        if interval in {"24h", "1d", "7d", "14d", "30d", "90d", "365d"}:
            start_date = now - timedelta(days=365) + timedelta(seconds=60)
        elif interval in {"1h", "2h", "3h", "6h", "12h"}:
            start_date = now - timedelta(days=1) + timedelta(seconds=60)
        else:
            raise ValueError("Invalid interval format")

        if existing_timestamp:
            start_date_existing = datetime.fromtimestamp(
                existing_timestamp[-1] + 2 * intervals, tz=timezone.utc
            )
            if start_date < start_date_existing:
                start_date = start_date_existing

        return start_date.strftime("%Y-%m-%dT%H:%M:%SZ")

    def get_history_price(
        self, currency: str, existing_timestamp: Optional[list[float]] = None
    ) -> list[dict[str, Any]]:
        if self.api_client is None:
            return []
        start_date = self.calculate_start_date(
            self.interval, existing_timestamp=existing_timestamp
        )
        timeseries = self.api_client.historical(
            self.whichcoin,
            quotes="USD",
            interval=self.interval,
            start=start_date,
        )
        return timeseries

    def update_price_history(self, currency: str) -> None:
        """Fetch historical prices from CoinPaprika."""
        if self.api_client is None:
            return
        logger.info(f"Getting historical data for a {self.interval} interval")
        existing_timestamp = self.price_history.get_timestamp_list()
        timeseries = self.get_history_price(
            currency, existing_timestamp=existing_timestamp
        )
        for price in timeseries:
            dt = datetime.strptime(price["timestamp"], "%Y-%m-%dT%H:%M:%SZ").replace(
                tzinfo=timezone.utc
            )
            self.price_history.add_price(dt, price["price"])

    def get_ohlcv(self, currency: str, existing_timestamp=None) -> pd.DataFrame:
        if self.api_client is None:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])
        start_date = self.calculate_start_date(
            "1h", existing_timestamp=existing_timestamp
        )
        raw_ohlcv = self.api_client.ohlcv(self.whichcoin, start=start_date)
        timeseries = [
            {
                "time": datetime.strptime(
                    ohlcv["time_open"], "%Y-%m-%dT%H:%M:%SZ"
                ).replace(tzinfo=timezone.utc),
                "ohlcv": [
                    ohlcv["open"],
                    ohlcv["high"],
                    ohlcv["low"],
                    ohlcv["close"],
                    ohlcv["volume"],
                ],
            }
            for ohlcv in raw_ohlcv
            if (
                datetime.strptime(
                    raw_ohlcv[-1]["time_open"], "%Y-%m-%dT%H:%M:%SZ"
                ).replace(tzinfo=timezone.utc)
                - datetime.strptime(ohlcv["time_open"], "%Y-%m-%dT%H:%M:%SZ").replace(
                    tzinfo=timezone.utc
                )
            ).days
            <= self.days_ago
        ]

        df = pd.DataFrame(
            [ohlcv["ohlcv"] for ohlcv in timeseries],
            columns=["Open", "High", "Low", "Close", "Volume"],
        )
        df.index = [ohlc["time"] for ohlc in timeseries]

        if existing_timestamp:
            cutoff = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
            df = df[df.index > cutoff]

        return df

    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        """Fetch OHLC data based on the number of days ago."""
        ohlcv_df = self.get_ohlcv(currency, existing_timestamp)
        ohlc_df = ohlcv_df.drop(columns=["Volume"])
        return ohlc_df



import unittest
from datetime import datetime, timezone
from unittest.mock import patch

import pandas as pd

from btcpriceticker.coingecko import CoinGecko
from btcpriceticker.coinpaprika import CoinPaprika
from btcpriceticker.mempool import Mempool
from btcpriceticker.price import Price


class TestPrice(unittest.TestCase):
    @patch.object(CoinGecko, "get_current_price")
    @patch.object(CoinGecko, "get_ohlcv")
    @patch.object(CoinGecko, "get_history_price")
    @patch("btcpriceticker.price_timeseries.PriceTimeSeries.get_price_list")
    def test_refresh_with_coingecko(
        self,
        mock_get_price_list,
        mock_get_history_price,
        mock_get_ohlcv,
        mock_get_current_price,
    ):
        # Mock responses
        mock_get_current_price.side_effect = lambda currency: {
            "usd": 50000,
            "eur": 42000,
        }[currency]
        mock_get_ohlcv.return_value = pd.DataFrame(
            [[49000, 51000, 48000, 50000], [50010, 51500, 48500, 50500]],
            columns=["Open", "High", "Low", "Close"],
            index=pd.date_range(datetime.now(timezone.utc), periods=2, freq="h"),
        )
        mock_get_history_price.return_value = {"prices": [[1000, 40000], [2000, 50000]]}
        # Mock price_list to return a non-empty list
        mock_get_price_list.return_value = [40000, 50000]

        price_instance = Price(fiat="eur", days_ago=1, service="coingecko")
        self.assertTrue(price_instance.refresh())

        self.assertTrue(price_instance.price["usd"])
        self.assertTrue(price_instance.price["fiat"])
        self.assertTrue(price_instance.price["sat_usd"])
        self.assertTrue(price_instance.price["sat_fiat"])
        self.assertTrue(price_instance.timeseries_stack)

    @patch.object(CoinPaprika, "get_current_price")
    @patch.object(CoinPaprika, "update_price_history")
    @patch.object(CoinPaprika, "get_history_price")
    @patch.object(CoinPaprika, "get_ohlcv")
    @patch("btcpriceticker.price_timeseries.PriceTimeSeries.get_price_list")
    def test_refresh_with_paprika(
        self,
        mock_get_price_list,
        mock_get_ohlcv,
        mock_get_history_price,
        mock_update_price_history,
        mock_get_current_price,
    ):
        # Mock responses
        mock_get_current_price.side_effect = lambda currency: {
            "USD": 50000,
            "EUR": 42000,
        }[currency]

        # Mock price history methods
        mock_update_price_history.return_value = None
        mock_get_history_price.return_value = [
            {"timestamp": "2023-01-01T12:00:00Z", "price": 42000}
        ]
        mock_get_ohlcv.return_value = pd.DataFrame(
            [[41000, 43000, 40000, 42000]],
            columns=["Open", "High", "Low", "Close"],
            index=pd.date_range(datetime.now(timezone.utc), periods=1, freq="h"),
        )
        # Mock price_list to return a non-empty list
        mock_get_price_list.return_value = [40000, 50000]

        price_instance = Price(
            fiat="eur", days_ago=1, service="coinpaprika", enable_timeseries=True
        )

        self.assertTrue(price_instance.refresh())

        self.assertTrue(price_instance.price["usd"])
        self.assertTrue(price_instance.price["fiat"])
        self.assertTrue(price_instance.price["sat_usd"])
        self.assertTrue(price_instance.price["sat_fiat"])
        self.assertTrue(price_instance.timeseries_stack)

    @patch.object(Mempool, "get_current_price")
    @patch.object(Mempool, "update_price_history")
    @patch.object(Mempool, "get_history_price")
    @patch.object(Mempool, "get_ohlcv")
    @patch("btcpriceticker.price_timeseries.PriceTimeSeries.get_price_list")
    def test_refresh_with_mempool(
        self,
        mock_get_price_list,
        mock_get_ohlcv,
        mock_get_history_price,
        mock_update_price_history,
        mock_get_current_price,
    ):
        # Mock responses
        mock_get_current_price.side_effect = lambda currency: {
            "USD": 50000,
            "EUR": 42000,
        }[currency]

        # Mock price history methods
        mock_update_price_history.return_value = None
        mock_get_history_price.return_value = [(datetime.now(timezone.utc), 42000)]
        mock_get_ohlcv.return_value = pd.DataFrame(
            [[41000, 43000, 40000, 42000]],
            columns=["Open", "High", "Low", "Close"],
            index=pd.date_range(datetime.now(timezone.utc), periods=1, freq="h"),
        )
        # Mock price_list to return a non-empty list
        mock_get_price_list.return_value = [40000, 50000]

        # Create Price instance with both timeseries and OHLCV enabled
        price_instance = Price(
            fiat="eur",
            days_ago=1,
            service="mempool",
            enable_timeseries=True,
            enable_ohlcv=True,
        )

        self.assertTrue(price_instance.refresh())

        # Verify price data was properly set
        self.assertTrue(price_instance.price["usd"])
        self.assertTrue(price_instance.price["fiat"])
        self.assertTrue(price_instance.price["sat_usd"])
        self.assertTrue(price_instance.price["sat_fiat"])
        self.assertTrue(price_instance.timeseries_stack)

    @patch.object(CoinGecko, "get_current_price")
    @patch.object(CoinGecko, "update_price_history")
    @patch.object(CoinGecko, "get_history_price")
    def test_get_price_now(
        self, mock_get_history_price, mock_update_price_history, mock_get_current_price
    ):
        # Mock the necessary methods
        mock_get_history_price.return_value = {"prices": [[1000, 40000], [2000, 50000]]}
        mock_update_price_history.return_value = None
        mock_get_current_price.side_effect = lambda currency: {
            "usd": 50000,
            "eur": 42000,
        }[currency]

        price_instance = Price(fiat="eur", days_ago=1, service="coingecko")
        price_instance.refresh()

        self.assertTrue(price_instance.get_price_now())

    def test_set_days_ago(self):
        price_instance = Price(fiat="eur", days_ago=1)
        price_instance.set_days_ago(7)
        self.assertEqual(price_instance.days_ago, 7)


if __name__ == "__main__":
    unittest.main()



from .binance import Binance
from .bit2me import Bit2Me
from .bitvavo import Bitvavo
from .coinbase import Coinbase
from .coingecko import CoinGecko
from .coinpaprika import CoinPaprika
from .kraken import Kraken
from .mempool import Mempool
from .price import Price

__all__ = [
    "Price",
    "CoinGecko",
    "CoinPaprika",
    "Kraken",
    "Mempool",
    "Binance",
    "Coinbase",
    "Bit2Me",
    "Bitvavo",
]



from datetime import datetime, timedelta, timezone
from typing import Optional

import pandas as pd


class PriceTimeSeries:
    def __init__(self):
        self.data = pd.DataFrame(columns=["timestamp", "price"])

    def add_price(self, timestamp: datetime, price: float) -> None:
        """Add a new price point to the time series."""
        new_data = pd.DataFrame([[timestamp, price]], columns=["timestamp", "price"])
        if not self.data.empty:
            self.data = pd.concat([self.data, new_data], ignore_index=True)
        else:
            self.data = new_data.reset_index(drop=True)

    def get_price_list(self, days: Optional[int] = None) -> list[float]:
        """Return the time series as a list of (timestamp, price) tuples,
        optionally filtered by the last `days` days."""
        if days is not None:
            cutoff = datetime.now(timezone.utc) - timedelta(days=days)
            filtered_data = self.data[self.data["timestamp"] >= cutoff]
        else:
            filtered_data = self.data
        return filtered_data.loc[:, "price"].values.tolist()

    def get_timestamp_list(self, days: Optional[int] = None) -> list[float]:
        """Return the time series as a list of (timestamp, price) tuples,
        optionally filtered by the last `days` days."""
        if days is not None:
            cutoff = datetime.now(timezone.utc) - timedelta(days=days)
            filtered_data = self.data[self.data["timestamp"] >= cutoff]
        else:
            filtered_data = self.data
        return (filtered_data["timestamp"].astype("int64") / 1e9).values.tolist()

    def get_data(self, days: Optional[int] = None) -> pd.DataFrame:
        """Return the time series as a Pandas DataFrame,
        optionally filtered by the last `days` days."""
        if days is not None:
            cutoff = datetime.now(timezone.utc) - timedelta(days=days)
            return self.data[self.data["timestamp"] >= cutoff]
        return self.data

    def append_dataframe(self, other_df: pd.DataFrame) -> None:
        """Append another dataframe containing timestamp
        and price columns to the time series."""
        if not {"timestamp", "price"}.issubset(other_df.columns):
            raise ValueError("DataFrame must contain 'timestamp' and 'price' columns")
        if other_df.empty:
            return
        if not self.data.empty:
            self.data = pd.concat([self.data, other_df], ignore_index=True)
        else:
            self.data = other_df.reset_index(drop=True)
        self.data = self.data.sort_values(by="timestamp").reset_index(drop=True)

    def get_percentage_change(self, days: int) -> Optional[float]:
        """Return the percentage change in price from the past `days` days to now."""
        cutoff = datetime.now(timezone.utc) - timedelta(days=days)
        past_data = self.data[self.data["timestamp"] >= cutoff]
        if past_data.empty:
            return None
        past_price = past_data.iloc[0]["price"]
        current_price = self.data.iloc[-1]["price"]
        return ((current_price - past_price) / past_price) * 100 if past_price else None

    def resample_to_ohlcv(self, time_frame: str) -> pd.DataFrame:
        """
        Convert a dataframe with timestamp and price into an OHLCV chart.

        Parameters:
            time_frame (str): Pandas-compatible resampling
            period (e.g., '1h', '1d', '15t')

        Returns:
            pd.DataFrame: DataFrame with columns ["Open", "High",
            "Low", "Close", "Volume"]
        """
        if self.data.empty:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])

        df = self.data.copy()
        df["timestamp"] = pd.to_datetime(df["timestamp"], utc=True)
        df.set_index("timestamp", inplace=True)

        ohlc = df["price"].resample(time_frame).agg(["first", "max", "min", "last"])
        ohlc.columns = ["Open", "High", "Low", "Close"]
        volume = df["price"].resample(time_frame).count().rename("Volume")

        ohlcv_df = pd.concat([ohlc, volume], axis=1)
        return ohlcv_df.dropna(how="all")



import logging
from datetime import datetime, timedelta, timezone
from typing import Any, Optional

import pandas as pd

from .service import Service

logger = logging.getLogger(__name__)

MEMPOOL_MODULE = None
try:
    from pymempool.api import MempoolAPI

    MEMPOOL_MODULE = "pymempool"
except ImportError:
    pass


class Mempool(Service):
    def __init__(
        self,
        fiat,
        interval="1h",
        days_ago=1,
        enable_ohlc=False,
        enable_timeseries=False,
        enable_ohlcv=False,
    ):
        self.api_client: Optional[Any] = MempoolAPI() if MEMPOOL_MODULE else None
        self.initialize(
            fiat,
            interval=interval,
            days_ago=days_ago,
            enable_ohlc=enable_ohlc,
            enable_timeseries=enable_timeseries,
            enable_ohlcv=enable_ohlcv,
        )
        self.name = "mempool"

    def get_current_price(self, currency="USD") -> Optional[float]:
        """Fetch the current price from Mempool."""
        if not self.api_client:
            return None
        try:
            ticker = self.api_client.get_price()
            return float(ticker[currency.upper()])
        except Exception as e:
            logger.exception(f"Failed to fetch current price: {e}")
            return None

    def interval_to_seconds(self) -> int:
        """Convert a time interval string to seconds."""
        unit_multipliers = {"m": 60, "h": 3600, "d": 86400}

        try:
            value, unit = int(self.interval[:-1]), self.interval[-1]
            if unit in unit_multipliers:
                return value * unit_multipliers[unit]
            else:
                raise ValueError(f"Invalid interval format {self.interval}")
        except (ValueError, IndexError) as e:
            raise ValueError(f"Invalid interval format {self.interval}") from e

    def calculate_start_date(self):
        now = datetime.utcnow()
        return now - timedelta(days=self.days_ago)

    def calculate_time_vector(self, existing_timestamp=None):
        """Generate timestamps based on the interval."""
        now = datetime.utcnow()
        intervals = self.interval_to_seconds()

        if not existing_timestamp:
            start_time = self.calculate_start_date()
        else:
            start_time = datetime.utcfromtimestamp(
                existing_timestamp[-1] + 2 * intervals
            )
        time_vector = list(
            range(
                int(start_time.timestamp()),
                int(now.timestamp()),
                intervals,
            )
        )

        return time_vector

    def get_history_price(
        self, currency, existing_timestamp=None
    ) -> list[tuple[datetime, float]]:
        history_prices: list[tuple[datetime, float]] = []
        if self.api_client is None:
            return history_prices
        time_vector = self.calculate_time_vector(existing_timestamp=existing_timestamp)
        for timestamp in time_vector:
            price = self.api_client.get_historical_price(
                currency=currency.upper(), timestamp=timestamp
            )
            price_value = float(price["prices"][0][currency.upper()])
            history_prices.append(
                (datetime.fromtimestamp(timestamp, tz=timezone.utc), price_value)
            )
        return history_prices

    def update_price_history(self, currency):
        """Fetch historical prices from Mempool."""
        logger.info(f"Getting historical data for a {self.interval} interval")
        existing_timestamp = self.price_history.get_timestamp_list()
        history_prices = self.get_history_price(
            currency, existing_timestamp=existing_timestamp
        )
        for price_time, price_value in history_prices:
            self.price_history.add_price(price_time, price_value)

    def get_ohlcv(self, currency, existing_timestamp=None):
        df = self.price_history.data.copy()
        df["timestamp"] = pd.to_datetime(df["timestamp"])
        df.set_index("timestamp", inplace=True)

        ohlcv_df = df["price"].resample("1h").ohlcv()
        if existing_timestamp:
            cutoff = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
            ohlcv_df = ohlcv_df[ohlcv_df.index > cutoff]
        return ohlcv_df

    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        """Fetch OHLC data based on the number of days ago."""
        ohlcv_df = self.get_ohlcv(currency, existing_timestamp)
        ohlc_df = ohlcv_df.drop(columns=["Volume"])
        return ohlc_df



from datetime import datetime, timedelta, timezone

import pandas as pd
import pytest

from btcpriceticker.price_timeseries import PriceTimeSeries


class TestPriceTimeSeries:
    def setup_method(self):
        self.series = PriceTimeSeries()

    def test_add_price_and_get_price_list_with_days_filter(self):
        old_timestamp = datetime.now(timezone.utc) - timedelta(days=2)
        recent_timestamp = datetime.now(timezone.utc)

        self.series.add_price(old_timestamp, 100.0)
        self.series.add_price(recent_timestamp, 200.0)

        all_prices = self.series.get_price_list()
        filtered_prices = self.series.get_price_list(days=1)

        assert all_prices == [100.0, 200.0]
        assert filtered_prices == [200.0]

    def test_get_timestamp_list_filters_by_days(self):
        old_timestamp = datetime.now(timezone.utc) - timedelta(days=3)
        recent_timestamp = datetime.now(timezone.utc)

        self.series.add_price(old_timestamp, 100.0)
        self.series.add_price(recent_timestamp, 200.0)

        timestamps = self.series.get_timestamp_list(days=1)

        assert len(timestamps) == 1
        assert isinstance(timestamps[0], float)

    def test_append_dataframe_combines_and_sorts(self):
        base_time = datetime.now(timezone.utc)
        self.series.add_price(base_time, 100.0)

        other_df = pd.DataFrame(
            {
                "timestamp": [
                    base_time - timedelta(hours=1),
                    base_time + timedelta(hours=1),
                ],
                "price": [90.0, 110.0],
            }
        )

        self.series.append_dataframe(other_df)

        prices = self.series.get_price_list()
        assert prices == [90.0, 100.0, 110.0]

    def test_append_dataframe_invalid_columns_raises(self):
        with pytest.raises(ValueError, match="must contain 'timestamp' and 'price'"):
            self.series.append_dataframe(pd.DataFrame({"foo": [1], "bar": [2]}))

    def test_get_percentage_change(self):
        base_time = datetime.now(timezone.utc)
        self.series.add_price(base_time - timedelta(days=1), 100.0)
        self.series.add_price(base_time, 110.0)

        change = self.series.get_percentage_change(days=2)

        assert change == pytest.approx(10.0)

    def test_get_percentage_change_returns_none_when_insufficient_data(self):
        change = self.series.get_percentage_change(days=1)

        assert change is None

    def test_resample_to_ohlcv(self):
        base_time = datetime(2024, 1, 1, 12, 0, tzinfo=timezone.utc)
        self.series.add_price(base_time, 100.0)
        self.series.add_price(base_time + timedelta(minutes=10), 105.0)
        self.series.add_price(base_time + timedelta(minutes=20), 95.0)
        self.series.add_price(base_time + timedelta(hours=1), 120.0)

        ohlcv = self.series.resample_to_ohlcv("1h")

        assert list(ohlcv.columns) == ["Open", "High", "Low", "Close", "Volume"]
        assert len(ohlcv) == 2

        first_row = ohlcv.iloc[0]
        assert first_row["Open"] == 100.0
        assert first_row["High"] == 105.0
        assert first_row["Low"] == 95.0
        assert first_row["Close"] == 95.0
        assert first_row["Volume"] == 3

    def test_resample_to_ohlcv_empty(self):
        result = self.series.resample_to_ohlcv("1h")

        assert result.empty



from __future__ import annotations

import base64
import hashlib
import hmac
import json
import logging
import math
import os
import time
from datetime import datetime, timezone
from typing import Any

import pandas as pd
import requests

from .service import Service

logger = logging.getLogger(__name__)


class Bit2Me(Service):
    base_url = "https://gateway.bit2me.com"

    def __init__(
        self,
        fiat: str,
        base_asset: str = "BTC",
        interval: str = "1h",
        days_ago: int = 1,
        enable_ohlc: bool = True,
        enable_timeseries: bool = True,
        enable_ohlcv: bool = True,
    ) -> None:
        self.base_asset = base_asset.upper()
        self.session = requests.Session()
        self.api_key = os.getenv("BIT2ME_API_KEY")
        self.api_secret = os.getenv("BIT2ME_API_SECRET")
        self._fiat_rates: dict[str, float] = {}
        self._fiat_rates_timestamp = 0.0
        self.initialize(
            fiat,
            interval=interval,
            days_ago=days_ago,
            enable_ohlc=enable_ohlc,
            enable_timeseries=enable_timeseries,
            enable_ohlcv=enable_ohlcv,
        )
        self.name = "bit2me"

    def _build_headers(
        self, path_url: str, body: dict[str, Any] | None = None
    ) -> dict[str, str]:
        headers: dict[str, str] = {"Accept": "application/json"}
        if not self.api_key:
            return headers

        headers["x-api-key"] = self.api_key
        nonce = str(int(time.time() * 1000))
        headers["x-nonce"] = nonce

        if not self.api_secret:
            return headers

        message = f"{nonce}:{path_url}"
        if body:
            body_str = json.dumps(body, separators=(",", ":"))
            message += f":{body_str}"

        sha = hashlib.sha256()
        sha.update(message.encode("utf-8"))
        digest = sha.digest()
        signature = hmac.new(
            self.api_secret.encode("utf-8"), digest, hashlib.sha512
        ).digest()
        headers["api-signature"] = base64.b64encode(signature).decode("utf-8")
        return headers

    def _request(
        self, method: str, endpoint: str, params: dict[str, Any] | None = None
    ) -> Any:
        request = requests.Request(
            method,
            f"{self.base_url}{endpoint}",
            params=params if method.upper() == "GET" else None,
            json=params if method.upper() != "GET" else None,
        )
        prepared = self.session.prepare_request(request)
        body = params if method.upper() != "GET" else None
        headers = self._build_headers(prepared.path_url, body)
        prepared.headers.update(headers)
        try:
            response = self.session.send(prepared, timeout=10)
        except requests.RequestException as exc:  # pragma: no cover - network errors
            logger.exception("Bit2Me request error: %s", exc)
            raise RuntimeError("Failed to reach Bit2Me API") from exc
        if response.status_code >= 400:
            logger.warning(
                "Bit2Me API returned %s for %s", response.status_code, prepared.path_url
            )
            raise RuntimeError(
                f"Bit2Me API error {response.status_code}: {response.text}"
            )
        if not response.content:
            return None
        try:
            return response.json()
        except ValueError:
            return response.text

    def _map_chart_temporality(self) -> str | None:
        mapping = {
            "1h": "one-hour",
            "4h": "four-hours",
            "12h": "twelve-hours",
            "1d": "one-day",
            "1w": "one-week",
            "1m": "one-month",
            "3m": "three-months",
            "6m": "six-months",
            "1y": "one-year",
            "max": "all-time",
        }
        return mapping.get(self.interval.lower())

    def _timeframe_to_seconds(self, timeframe: str) -> int:
        table = {
            "1H": 3600,
            "4H": 4 * 3600,
            "12H": 12 * 3600,
            "1D": 24 * 3600,
            "1W": 7 * 24 * 3600,
            "1M": 30 * 24 * 3600,
            "1Y": 365 * 24 * 3600,
        }
        return table.get(timeframe.upper(), 0)

    def _get_usd_price(self) -> float | None:
        try:
            data = self._request("GET", f"/v3/currency/ticker/{self.base_asset}")
        except RuntimeError:
            return None

        if not isinstance(data, dict):
            return None

        usd_bucket = data.get("USD")
        if not isinstance(usd_bucket, dict):
            return None

        asset_entries = usd_bucket.get(self.base_asset)
        if not isinstance(asset_entries, list):
            return None

        for entry in asset_entries:
            if isinstance(entry, dict) and entry.get("price") is not None:
                try:
                    return float(entry["price"])
                except (TypeError, ValueError):
                    continue
        return None

    def _refresh_rates(self) -> None:
        try:
            payload = self._request("GET", "/v1/currency/rate", params={"type": "fiat"})
        except RuntimeError:
            return

        rates: dict[str, float] = {}
        if isinstance(payload, list):
            for entry in payload:
                if not isinstance(entry, dict):
                    continue
                fiat_section = entry.get("fiat")
                if not isinstance(fiat_section, dict):
                    continue
                for code, value in fiat_section.items():
                    try:
                        rates[code.upper()] = float(value)
                    except (TypeError, ValueError):
                        continue

        if rates:
            rates["USD"] = 1.0
            self._fiat_rates = rates
            self._fiat_rates_timestamp = time.time()

    def _extract_rate(self, payload: Any, currency: str) -> float | None:
        if not isinstance(payload, list):
            return None
        for entry in payload:
            if not isinstance(entry, dict):
                continue
            fiat_section = entry.get("fiat")
            if not isinstance(fiat_section, dict):
                continue
            raw_rate = fiat_section.get(currency.upper())
            if raw_rate is None:
                continue
            try:
                return float(raw_rate)
            except (TypeError, ValueError):
                continue
        return None

    def _get_fiat_rate(self, currency: str) -> float | None:
        code = currency.upper()
        if code == "USD":
            return 1.0

        now = time.time()
        if not self._fiat_rates or (now - self._fiat_rates_timestamp) > 300:
            self._refresh_rates()

        rate = self._fiat_rates.get(code)
        if rate is not None:
            return rate

        # Attempt one more refresh in case rates were missing
        self._refresh_rates()
        return self._fiat_rates.get(code)

    def _fetch_ohlc_point(
        self, timeframe: str, currency: str, target_dt: datetime
    ) -> tuple[datetime, dict[str, float]] | None:
        iso_time = target_dt.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"
        rate_time = str(int(target_dt.timestamp() * 1000))
        try:
            usd_values = self._request(
                "GET",
                f"/v1/currency/ohlca/{self.base_asset}",
                params={"timeframe": timeframe, "time": iso_time},
            )
        except RuntimeError:
            return None

        if not isinstance(usd_values, dict):
            return None

        try:
            rate_payload = self._request(
                "GET",
                "/v1/currency/rate",
                params={"type": "fiat", "time": rate_time},
            )
        except RuntimeError:
            return None

        rate_value = self._extract_rate(rate_payload, currency)
        if rate_value is None:
            logger.warning("Bit2Me ohlc rate missing for %s", currency)
            return None

        try:
            open_price = float(usd_values["open"]) * rate_value
            high_price = float(usd_values["high"]) * rate_value
            low_price = float(usd_values["low"]) * rate_value
            close_price = float(usd_values["close"]) * rate_value
        except (KeyError, TypeError, ValueError):
            return None

        return (
            target_dt,
            {
                "Open": open_price,
                "High": high_price,
                "Low": low_price,
                "Close": close_price,
            },
        )

    def get_current_price(self, currency: str) -> float | None:
        usd_price = self._get_usd_price()
        if usd_price is None:
            return None

        rate = self._get_fiat_rate(currency)
        if rate is None:
            logger.warning("Bit2Me rate for %s not found", currency)
            return usd_price if currency.upper() == "USD" else None

        return usd_price * rate

    def _chart(self, currency: str) -> list[list[Any]]:
        params = {"ticker": f"{self.base_asset}/{currency.upper()}"}
        temporality = self._map_chart_temporality()
        if temporality:
            params["temporality"] = temporality
        try:
            data = self._request("GET", "/v3/currency/chart", params=params)
        except RuntimeError:
            return []
        if isinstance(data, list):
            return data
        return []

    def get_history_price(
        self, currency: str, existing_timestamp: list[float] | None = None
    ) -> list[list[float]]:
        chart = self._chart(currency)
        if not chart:
            return []
        last_ts = existing_timestamp[-1] if existing_timestamp else None
        history: list[list[float]] = []
        for entry in chart:
            if not entry:
                continue
            try:
                timestamp_ms = float(entry[0])
                inverse_price = float(entry[1])
                multiplier = float(entry[2])
            except (TypeError, ValueError, IndexError):
                continue

            if inverse_price == 0:
                continue

            timestamp_sec = timestamp_ms / 1000
            if last_ts and timestamp_sec <= last_ts:
                continue

            price_value = (1.0 / inverse_price) * multiplier
            history.append([timestamp_ms, price_value])
        return history

    def update_price_history(self, currency: str) -> None:
        data = self.get_history_price(
            currency, existing_timestamp=self.price_history.get_timestamp_list()
        )
        for timestamp_ms, price in data:
            dt = datetime.fromtimestamp(timestamp_ms / 1000, tz=timezone.utc)
            self.price_history.add_price(dt, price)

    def get_ohlcv(
        self, currency: str, existing_timestamp: list[float] | None = None
    ) -> pd.DataFrame:
        ohlc_df = self.get_ohlc(currency, existing_timestamp)
        if ohlc_df.empty:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close", "Volume"])
        ohlcv_df = ohlc_df.copy()
        ohlcv_df["Volume"] = 0.0
        return ohlcv_df

    def get_ohlc(
        self, currency: str, existing_timestamp: list[float] | None = None
    ) -> pd.DataFrame:
        timeframe = self.interval.upper()
        interval_seconds = self._timeframe_to_seconds(timeframe)
        if interval_seconds <= 0:
            interval_seconds = 3600

        now_dt = datetime.now(timezone.utc)
        now_ts = int(now_dt.timestamp())
        now_ts -= now_ts % interval_seconds
        now_dt = datetime.fromtimestamp(now_ts, tz=timezone.utc)

        rows: list[tuple[datetime, dict[str, float]]] = []
        seen = {int(ts) for ts in (existing_timestamp or [])}

        if existing_timestamp:
            last_ts = int(existing_timestamp[-1])
            start_ts = last_ts + interval_seconds
        else:
            window_seconds = max(self.days_ago * 86400, interval_seconds)
            periods = max(1, math.ceil(window_seconds / interval_seconds))
            start_ts = now_ts - (periods - 1) * interval_seconds

        current_ts = start_ts
        while current_ts <= now_ts:
            if current_ts in seen:
                current_ts += interval_seconds
                continue
            target_dt = datetime.fromtimestamp(current_ts, tz=timezone.utc)
            point = self._fetch_ohlc_point(timeframe, currency, target_dt)
            if point is not None:
                ts, values = point
                int_ts = int(ts.timestamp())
                if int_ts not in seen:
                    rows.append((ts, values))
                    seen.add(int_ts)
            current_ts += interval_seconds

        if not rows:
            return pd.DataFrame(columns=["Open", "High", "Low", "Close"])

        rows.sort(key=lambda item: item[0])
        index = [ts for ts, _ in rows]
        data = [values for _, values in rows]
        df = pd.DataFrame(data, index=index)
        df.index.name = "timestamp"
        return df



import unittest
from unittest.mock import patch

from btcpriceticker.coingecko import CoinGecko


class TestCoinGecko(unittest.TestCase):
    @patch("btcpriceticker.coingecko.CoinGeckoAPI.get_coins_markets")
    def test_get_current_price(self, mock_get_coins_markets):
        mock_get_coins_markets.return_value = [{"current_price": 50000}]

        cg = CoinGecko("usd", whichcoin="bitcoin")
        price = cg.get_current_price("usd")
        self.assertEqual(price, 50000)

    @patch("btcpriceticker.coingecko.CoinGeckoAPI.get_exchanges_tickers_by_id")
    def test_get_exchange_usd_price(self, mock_get_exchanges_tickers_by_id):
        mock_get_exchanges_tickers_by_id.return_value = {
            "tickers": [{"target": "USD", "last": 50000}]
        }

        cg = CoinGecko("usd", whichcoin="bitcoin")
        price = cg.get_exchange_usd_price("binance")
        self.assertEqual(price, 50000)

    @patch("btcpriceticker.coingecko.CoinGeckoAPI.get_coin_market_chart_by_id")
    @patch("btcpriceticker.coingecko.CoinGecko.get_current_price")
    def test_get_history_price(
        self, mock_get_current_price, mock_get_coin_market_chart_by_id
    ):
        mock_get_coin_market_chart_by_id.return_value = {
            "prices": [[1609459200000, 40000], [1609545600000, 50000.0]]
        }
        mock_get_current_price.return_value = 50000.0

        cg = CoinGecko("usd", whichcoin="bitcoin", days_ago=1)
        history = cg.get_history_price("usd")
        self.assertEqual(
            history, {"prices": [[1609459200000, 40000], [1609545600000, 50000.0]]}
        )


if __name__ == "__main__":
    unittest.main()



from unittest.mock import MagicMock, patch

from typer.testing import CliRunner

from btcpriceticker import _version
from btcpriceticker.cli import app, main, state


class TestCLI:
    def setup_method(self):
        state["verbose"] = 3
        state["service"] = "mempool"
        self.runner = CliRunner()

    def test_version_module_exposes_metadata(self):
        assert isinstance(_version.__version__, str)
        assert isinstance(_version.version_tuple, tuple)

    def test_main_updates_state(self):
        main(verbose=1, service="kraken")

        assert state["verbose"] == 1
        assert state["service"] == "kraken"

    def test_price_command_uses_state_service(self):
        with patch("btcpriceticker.cli.Price") as mock_price:
            mock_instance = mock_price.return_value
            mock_instance.get_price_now.return_value = "123"

            result = self.runner.invoke(app, ["price", "eur"])

            assert result.exit_code == 0
            mock_price.assert_called_once_with(
                service="mempool",
                fiat="eur",
                enable_ohlc=False,
                enable_timeseries=False,
                enable_ohlcv=False,
            )
            assert "123" in result.stdout

    def test_history_command_prints_timeseries(self):
        with patch("btcpriceticker.cli.Price") as mock_price:
            mock_instance = mock_price.return_value
            mock_instance.timeseries = MagicMock()
            mock_instance.timeseries.data = "history-data"

            result = self.runner.invoke(app, ["history", "usd", "1h"])

            assert result.exit_code == 0
            assert "history-data" in result.stdout

    def test_ohlc_and_ohlcv_commands_use_flags(self):
        with patch("btcpriceticker.cli.Price") as mock_price:
            price_instance = mock_price.return_value
            price_instance.ohlc = "ohlc-data"
            price_instance.ohlcv = "ohlcv-data"

            result_ohlc = self.runner.invoke(app, ["ohlc", "usd", "1h"])
            result_ohlcv = self.runner.invoke(app, ["ohlcv", "usd", "1h"])

            assert result_ohlc.exit_code == 0
            assert result_ohlcv.exit_code == 0

            ohlc_call = mock_price.call_args_list[-2]
            ohlcv_call = mock_price.call_args_list[-1]

            assert ohlc_call.kwargs["enable_ohlc"] is True
            assert ohlc_call.kwargs["enable_ohlcv"] is False
            assert ohlcv_call.kwargs["enable_ohlc"] is False
            assert ohlcv_call.kwargs["enable_ohlcv"] is True

            assert "ohlc-data" in result_ohlc.stdout
            assert "ohlcv-data" in result_ohlcv.stdout

    def test_cli_arguments_update_state_before_commands(self):
        with patch("btcpriceticker.cli.Price") as mock_price:
            mock_price.return_value.get_price_now.return_value = "456"

            result = self.runner.invoke(
                app,
                ["--service", "kraken", "--verbose", "0", "price", "usd"],
            )

            assert result.exit_code == 0
            mock_price.assert_called_with(
                service="kraken",
                fiat="usd",
                enable_ohlc=False,
                enable_timeseries=False,
                enable_ohlcv=False,
            )
            assert state["service"] == "kraken"



import abc
from datetime import datetime, timezone
from typing import Any, Optional, Union

import pandas as pd

from .price_timeseries import PriceTimeSeries


class Service(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __init__(self, fiat):
        self.initialize(fiat)

    def initialize(
        self,
        fiat,
        interval="1h",
        days_ago=1,
        enable_ohlc=False,
        enable_timeseries=False,
        enable_ohlcv=False,
    ):
        self.interval = interval
        self.days_ago = days_ago
        self.name = ""
        self.fiat = fiat
        self.enable_ohlc = enable_ohlc
        self.enable_ohlcv = enable_ohlcv
        self.enable_timeseries = enable_timeseries
        self.ohlc: Union[pd.DataFrame, Any] = pd.DataFrame(
            columns=["Open", "High", "Low", "Close"]
        )

        self.ohlcv: Union[pd.DataFrame, Any] = pd.DataFrame(
            columns=["Open", "High", "Low", "Close", "Volume"]
        )
        self.price = {
            "usd": 0.0,
            "sat_usd": 0.0,
            "fiat": 0.0,
            "sat_fiat": 0.0,
            "timestamp": 0.0,
        }
        self.price_history = PriceTimeSeries()

    def get_name(self):
        return self.name

    def get_price(self):
        return self.price

    def _safe_get_current_price(self, currency: str) -> Optional[float]:
        variations = (currency, currency.upper(), currency.lower())
        for variant in variations:
            try:
                price = self.get_current_price(variant)
            except Exception:
                continue
            if price is not None:
                return price
        return None

    def update(self):
        now = datetime.now(timezone.utc)
        current_time = now.timestamp()

        # Get USD price, defaulting to 0.0 if None is returned
        usd_price = self._safe_get_current_price("USD")
        self.price["usd"] = usd_price if usd_price is not None else 0.0
        self.price["sat_usd"] = 1e8 / self.price["usd"] if self.price["usd"] else 0.0

        # Get fiat price, defaulting to 0.0 if None is returned
        fiat_price = self._safe_get_current_price(self.fiat)
        self.price["fiat"] = fiat_price if fiat_price is not None else 0.0
        self.price["sat_fiat"] = 1e8 / self.price["fiat"] if self.price["fiat"] else 0.0

        if self.enable_timeseries:
            self.update_price_history(self.fiat)
        else:
            self.append_current_price(self.price["fiat"])
        if self.enable_ohlcv:
            self.update_ohlcv(self.fiat)
        if self.enable_ohlc:
            self.update_ohlc(self.fiat)

        self.price["timestamp"] = current_time

    def append_current_price(self, current_price):
        now = datetime.now(timezone.utc)
        self.price_history.add_price(now, current_price)

    def get_price_list(self):
        return self.price_history.get_price_list(days=self.days_ago)

    def get_price_change(self):
        change_percentage = self.price_history.get_percentage_change(self.days_ago)
        if not change_percentage:
            return ""
        return f"{change_percentage:+.2f}%"

    def _get_ohlcv_timestamp_list(self) -> list[float]:
        if isinstance(self.ohlcv, pd.DataFrame) and not self.ohlcv.empty:
            if isinstance(self.ohlcv.index, pd.DatetimeIndex):
                timestamps = self.ohlcv.index.astype("int64") / 1e9
                return timestamps.tolist()
        return []

    def update_ohlcv(self, currency: str) -> None:
        existing_timestamp = self._get_ohlcv_timestamp_list()
        ohlcv_data = self.get_ohlcv(currency, existing_timestamp=existing_timestamp)

        if isinstance(ohlcv_data, pd.DataFrame):
            if ohlcv_data.empty:
                return
            new_data = ohlcv_data.sort_index()
            if isinstance(self.ohlcv, pd.DataFrame) and not self.ohlcv.empty:
                combined = pd.concat([self.ohlcv, new_data])
                combined = combined[~combined.index.duplicated(keep="last")]  # type: ignore[index]
                self.ohlcv = combined.sort_index()
            else:
                self.ohlcv = new_data
        else:
            self.ohlcv = ohlcv_data

    def update_ohlc(self, currency: str) -> None:
        existing_timestamp = self._get_ohlcv_timestamp_list()
        ohlc_data = self.get_ohlc(currency, existing_timestamp=existing_timestamp)

        if isinstance(ohlc_data, pd.DataFrame):
            if ohlc_data.empty:
                return
            new_data = ohlc_data.sort_index()
            if isinstance(self.ohlc, pd.DataFrame) and not self.ohlc.empty:
                combined = pd.concat([self.ohlc, new_data])
                combined = combined[~combined.index.duplicated(keep="last")]  # type: ignore[index]
                self.ohlc = combined.sort_index()
            else:
                self.ohlc = new_data
        else:
            self.ohlc = ohlc_data

    @abc.abstractmethod
    def get_current_price(self, currency) -> Optional[float]:
        pass

    @abc.abstractmethod
    def get_history_price(self, currency, existing_timestamp=None) -> Any:
        pass

    @abc.abstractmethod
    def update_price_history(self, currency) -> None:
        pass

    @abc.abstractmethod
    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        pass

    @abc.abstractmethod
    def get_ohlcv(self, currency, existing_timestamp=None) -> pd.DataFrame:
        pass



import unittest
from datetime import datetime, timedelta, timezone
from typing import Any, Optional
from unittest.mock import patch

import pandas as pd

from btcpriceticker.price_timeseries import PriceTimeSeries
from btcpriceticker.service import Service


class MockService(Service):
    """Mock implementation of the abstract Service class for testing."""

    def __init__(self, fiat="usd"):
        self.initialize(fiat)
        self.name = "mock_service"

    def get_current_price(self, currency) -> Optional[float]:
        # Mock implementation returns fixed values based on currency
        if currency.lower() == "usd":
            return 50000
        return 42000

    def get_history_price(self, currency, existing_timestamp=None) -> Any:
        # Mock implementation returns fixed history
        # Return a simple list of price values
        return [40000, 50000]

    def update_price_history(self, currency) -> None:
        # Mock implementation adds a fixed price
        now = datetime.now(timezone.utc)
        self.price_history.add_price(now, 50000)

    def get_ohlc(self, currency, existing_timestamp=None) -> pd.DataFrame:
        base_time = datetime.now(timezone.utc)
        if existing_timestamp:
            base_time = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
        index = pd.date_range(base_time + timedelta(hours=1), periods=1, freq="h")
        return pd.DataFrame(
            [[49000, 51000, 48000, 50000]],
            columns=["Open", "High", "Low", "Close"],
            index=index,
        )

    def get_ohlcv(self, currency, existing_timestamp=None) -> pd.DataFrame:
        base_time = datetime.now(timezone.utc)
        if existing_timestamp:
            base_time = datetime.fromtimestamp(existing_timestamp[-1], tz=timezone.utc)
        index = pd.date_range(base_time + timedelta(hours=1), periods=1, freq="h")
        return pd.DataFrame(
            [[49000, 51000, 48000, 50000, 12345]],
            columns=["Open", "High", "Low", "Close", "Volume"],
            index=index,
        )


class TestService(unittest.TestCase):
    def setUp(self):
        self.service = MockService("eur")

    def test_initialize(self):
        """Test initialization of the Service class."""
        self.assertEqual(self.service.fiat, "eur")
        self.assertEqual(self.service.days_ago, 1)
        self.assertEqual(self.service.name, "mock_service")
        self.assertFalse(self.service.enable_ohlcv)
        self.assertFalse(self.service.enable_ohlc)
        self.assertFalse(self.service.enable_timeseries)
        self.assertEqual(self.service.price["usd"], 0)
        self.assertEqual(self.service.price["fiat"], 0)
        self.assertIsInstance(self.service.price_history, PriceTimeSeries)

    def test_get_name(self):
        """Test get_name method."""
        self.assertEqual(self.service.get_name(), "mock_service")

    def test_get_price(self):
        """Test get_price method."""
        self.assertEqual(self.service.get_price(), self.service.price)

    def test_update(self):
        """Test update method."""
        self.service.update()

        # Check USD and fiat prices are updated
        self.assertEqual(self.service.price["usd"], 50000)
        self.assertEqual(self.service.price["fiat"], 42000)

        # Check sat conversions
        self.assertEqual(self.service.price["sat_usd"], 1e8 / 50000)
        self.assertEqual(self.service.price["sat_fiat"], 1e8 / 42000)

        # Check timestamp is updated
        self.assertGreater(self.service.price["timestamp"], 0)

    def test_update_with_zero_price(self):
        """Test update method with zero price handling."""
        with patch.object(MockService, "get_current_price", return_value=0):
            self.service.update()

            # Check sat values are zero when price is zero
            self.assertEqual(self.service.price["sat_usd"], 0)
            self.assertEqual(self.service.price["sat_fiat"], 0)

    def test_append_current_price(self):
        """Test append_current_price method."""
        initial_count = len(self.service.price_history.get_price_list())
        self.service.append_current_price(50000)
        new_count = len(self.service.price_history.get_price_list())

        # Verify a price was added to the history
        self.assertEqual(new_count, initial_count + 1)

    def test_get_price_list(self):
        """Test get_price_list method."""
        # Mock the price_history.get_price_list method to avoid timezone issues
        with patch.object(PriceTimeSeries, "get_price_list", return_value=[50000]):
            price_list = self.service.get_price_list()

            # Verify we get a list of prices
            self.assertIsInstance(price_list, list)
            self.assertEqual(price_list, [50000])

    def test_get_price_change(self):
        """Test get_price_change method."""
        # Mock the get_percentage_change method to return a known value
        with patch.object(PriceTimeSeries, "get_percentage_change", return_value=5.25):
            change = self.service.get_price_change()
            self.assertEqual(change, "+5.25%")

        # Test with no change
        with patch.object(PriceTimeSeries, "get_percentage_change", return_value=None):
            change = self.service.get_price_change()
            self.assertEqual(change, "")

    def test_update_with_timeseries_enabled(self):
        """Test update with timeseries enabled."""
        self.service.enable_timeseries = True

        # Need to patch update_price_history to verify it's called
        with patch.object(MockService, "update_price_history") as mock_update:
            self.service.update()
            mock_update.assert_called_once_with("eur")

    def test_update_with_ohlcv_enabled(self):
        """Test update with OHLCV enabled."""
        self.service.enable_ohlcv = True

        # Need to patch get_ohlcv to verify it's called
        with patch.object(MockService, "get_ohlcv") as mock_ohlcv:
            mock_ohlcv.return_value = pd.DataFrame(
                [[49000, 51000, 48000, 50000, 12345]],
                columns=["Open", "High", "Low", "Close", "Volume"],
                index=pd.date_range(datetime.now(timezone.utc), periods=1, freq="h"),
            )
            self.service.update()
            mock_ohlcv.assert_called_once()
            args, kwargs = mock_ohlcv.call_args
            self.assertEqual(args[0], "eur")
            self.assertIn("existing_timestamp", kwargs)
            self.assertIsInstance(self.service.ohlcv, pd.DataFrame)
            self.assertFalse(self.service.ohlcv.empty)

    def test_update_with_ohlc_enabled(self):
        """Test update with OHLC enabled."""
        self.service.enable_ohlc = True

        with patch.object(MockService, "get_ohlc") as mock_ohlc:
            mock_ohlc.return_value = pd.DataFrame(
                [[49000, 51000, 48000, 50000]],
                columns=["Open", "High", "Low", "Close"],
                index=pd.date_range(datetime.now(timezone.utc), periods=1, freq="h"),
            )
            self.service.update()
            mock_ohlc.assert_called_once()
            args, kwargs = mock_ohlc.call_args
            self.assertEqual(args[0], "eur")
            self.assertIn("existing_timestamp", kwargs)
            self.assertIsInstance(self.service.ohlc, pd.DataFrame)
            self.assertFalse(self.service.ohlc.empty)

    def test_update_ohlcv_accumulates(self):
        self.service.enable_ohlcv = True
        with patch.object(MockService, "get_ohlcv") as mock_ohlcv:
            base_time = datetime.now(timezone.utc)
            first_df = pd.DataFrame(
                [[49000, 51000, 48000, 50000, 12345]],
                columns=["Open", "High", "Low", "Close", "Volume"],
                index=pd.date_range(base_time, periods=1, freq="h"),
            )
            second_df = pd.DataFrame(
                [[50010, 51500, 48500, 50500, 23456]],
                columns=["Open", "High", "Low", "Close", "Volume"],
                index=pd.date_range(
                    base_time + timedelta(hours=1), periods=1, freq="h"
                ),
            )
            mock_ohlcv.side_effect = [first_df, second_df]

            self.service.update()
            self.service.update()

            self.assertEqual(len(self.service.ohlcv), 2)
            self.assertTrue((self.service.ohlcv.iloc[0] == first_df.iloc[0]).all())
            self.assertTrue((self.service.ohlcv.iloc[1] == second_df.iloc[0]).all())


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch

from btcpriceticker.mempool import Mempool


class TestMempool(unittest.TestCase):
    @patch("pymempool.MempoolAPI.get_price")
    def test_get_current_price(self, mock_for_coin):
        mock_for_coin.return_value = {"USD": 50000}

        m = Mempool("USD")
        price = m.get_current_price("USD")
        self.assertEqual(price, 50000)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import MagicMock, patch

from btcpriceticker.kraken import Kraken


class TestKraken(unittest.TestCase):
    @patch("btcpriceticker.kraken.ccxt.kraken")
    def test_get_current_price(self, mock_kraken):
        exchange = MagicMock()
        exchange.fetch_ticker.return_value = {"last": 50000}
        mock_kraken.return_value = exchange

        kraken_service = Kraken("EUR")
        price = kraken_service.get_current_price("USD")

        self.assertEqual(price, 50000.0)

    @patch("btcpriceticker.kraken.ccxt.kraken")
    def test_update_price_history(self, mock_kraken):
        exchange = MagicMock()
        exchange.fetch_ohlcv.return_value = [
            [1609459200000, 29000, 29500, 28500, 29200, 10.5],
            [1609462800000, 29200, 29600, 28900, 29450, 8.3],
        ]
        mock_kraken.return_value = exchange

        kraken_service = Kraken("EUR", interval="1h", enable_timeseries=True)
        kraken_service.update_price_history("EUR")

        self.assertGreater(len(kraken_service.price_history.data), 0)
        exchange.fetch_ohlcv.assert_called()

    @patch("btcpriceticker.kraken.ccxt.kraken")
    def test_get_ohlc_with_existing_timestamp(self, mock_kraken):
        exchange = MagicMock()
        exchange.fetch_ohlcv.return_value = [
            [1609459200000, 29000, 29500, 28500, 29200, 10.5],
            [1609462800000, 29200, 29600, 28900, 29450, 8.3],
        ]
        mock_kraken.return_value = exchange

        kraken_service = Kraken("EUR", interval="1h", enable_ohlcv=True)
        existing = [1609455600.0]
        df = kraken_service.get_ohlcv("EUR", existing_timestamp=existing)

        exchange.fetch_ohlcv.assert_called()
        self.assertFalse(df.empty)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch

from btcpriceticker.coinpaprika import CoinPaprika


class TestCoinpaprika(unittest.TestCase):
    @patch("coinpaprika.client.Client.ticker")
    def test_get_current_price(self, mock_for_coin):
        mock_for_coin.return_value = {"quotes": {"USD": {"price": 50000}}}

        cp = CoinPaprika("USD", whichcoin="btc-bitcoin")
        price = cp.get_current_price("USD")
        self.assertEqual(price, 50000)

    @patch("coinpaprika.client.Client.exchange_markets")
    def test_get_exchange_usd_price(self, mock_markets):
        mock_markets.return_value = [
            {"pair": "BTC/USDC", "quotes": {"USD": {"price": 50000}}}
        ]

        cp = CoinPaprika("USD", whichcoin="btc-bitcoin")
        price = cp.get_exchange_usd_price("binance", "BTC/USDC")
        self.assertEqual(price, 50000)


if __name__ == "__main__":
    unittest.main()



import logging
from typing import TypedDict

import typer
from rich.console import Console

from btcpriceticker.price import Price

log = logging.getLogger(__name__)
app = typer.Typer()
console = Console()


class State(TypedDict):
    verbose: int
    service: str


state: State = {"verbose": 3, "service": "mempool"}


@app.command(help="Show the latest BTC price converted into the given fiat symbol.")
def price(
    symbol: str = typer.Argument(..., help="Fiat currency code, e.g. 'EUR'."),
    show_symbol: bool = typer.Option(
        False, help="Displays the fiat currency symbol alongside the price."
    ),
    as_float: bool = typer.Option(False, help="Returns the price as float."),
):
    p = Price(
        service=state["service"],
        fiat=symbol,
        enable_ohlc=False,
        enable_timeseries=False,
        enable_ohlcv=False,
    )
    p.refresh()
    if not as_float and show_symbol:
        price = p.get_price_now()
        symbol = p.get_fiat_symbol()
        print(f"{price} {symbol}")
    elif not as_float and not show_symbol:
        price = p.get_price_now()
        print(price)
    else:
        price = p.get_fiat_price()
        print(price)


@app.command(help="Display historical BTC prices for the requested interval.")
def history(
    symbol: str = typer.Argument(..., help="Fiat currency code, e.g. 'EUR'."),
    interval: str = typer.Argument(..., help="Sampling interval, e.g. '1h'."),
    days_ago: int = typer.Option(
        1, help="Number of days of data to pull counting back from now."
    ),
):
    p = Price(
        service=state["service"],
        days_ago=days_ago,
        fiat=symbol,
        interval=interval,
        enable_ohlc=False,
        enable_timeseries=True,
        enable_ohlcv=False,
    )
    p.refresh()
    print(p.timeseries.data)


@app.command(help="Retrieve OHLC candles converted into the requested fiat.")
def ohlc(
    symbol: str = typer.Argument(..., help="Fiat currency code, e.g. 'EUR'."),
    interval: str = typer.Argument(..., help="Candle interval, e.g. '12h'."),
    days_ago: int = typer.Option(
        1, help="Number of past days to cover when fetching candles."
    ),
):
    p = Price(
        service=state["service"],
        days_ago=days_ago,
        fiat=symbol,
        interval=interval,
        enable_ohlc=True,
        enable_timeseries=True,
        enable_ohlcv=False,
    )
    p.refresh()
    print(p.ohlc)


@app.command(help="Retrieve OHLCV candles (with synthetic volume) for BTC.")
def ohlcv(
    symbol: str = typer.Argument(..., help="Fiat currency code, e.g. 'EUR'."),
    interval: str = typer.Argument(..., help="Candle interval, e.g. '1h'."),
    days_ago: int = typer.Option(
        1, help="Number of past days to cover when fetching candles."
    ),
):
    p = Price(
        service=state["service"],
        days_ago=days_ago,
        fiat=symbol,
        interval=interval,
        enable_ohlc=False,
        enable_timeseries=True,
        enable_ohlcv=True,
    )
    p.refresh()
    print(p.ohlcv)


@app.callback()
def main(
    verbose: int = typer.Option(
        3,
        help="Verbosity level: 0=critical, 1=error, 2=warn, 3=info, 4=debug.",
    ),
    service: str = typer.Option(
        "mempool",
        help="Service backend to use, e.g. 'bit2me', 'binance', or 'mempool'.",
    ),
):
    """BTC price utilities for multiple exchange backends."""
    # Logging
    state["verbose"] = verbose
    state["service"] = service
    log = logging.getLogger(__name__)
    verbosity = ["critical", "error", "warn", "info", "debug"][int(min(verbose, 4))]
    log.setLevel(getattr(logging, verbosity.upper()))
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    ch = logging.StreamHandler()
    ch.setLevel(getattr(logging, verbosity.upper()))
    ch.setFormatter(formatter)
    log.addHandler(ch)


if __name__ == "__main__":
    app()
